#include <time.h>
#include "fmt.h"
#include "env.h"
#include "stralloc.h"
#include "byte.h"
#include "strerr.h"
#include "openreadclose.h"
#include "str.h"
#include "scan.h"
#include "cookie.h"
#include "qdated.h"

#define FATAL "qdated-check: fatal: "
#define INFO "qdated-check: info: "

void die_nomem() { strerr_die2x(1,FATAL,"out of memory."); }

char str_extdate[FMT_ULONG];
char hash[COOKIE];
char *home;
char *ext;
unsigned long maxage = 604800; /* 1 week */
long extdate;
stralloc fn = {0};
stralloc key = {0};

int main(int argc,char **argv)
{
  long now = time((time_t *) 0);
  char *x;
  unsigned long u;

  if (argv++ && *argv)
    scan_ulong(*argv,&maxage);
  if (maxage < 1000) maxage *= 86400;

  home = env_get("HOME");
  ext = env_get("DEFAULT");

  if (!home)
    strerr_die2x(1,FATAL,"can't figure out where HOME is.");
  if (!ext)
    strerr_die2x(1,FATAL,"DEFAULT isn't set.");

  if (!stralloc_copys(&fn,home)) die_nomem();
  if (!stralloc_cats(&fn,"/.qdated-key")) die_nomem();
  if (!stralloc_0(&fn)) die_nomem();

  if (!openreadclose(fn.s,&key,32))
    strerr_die2x(0,INFO,"can't open key file--allowing address.");

  x = ext;
  x += scan_ulong(x,&u);
  extdate = u;
  if (extdate + BASETIME < now - maxage)
    strerr_die2x(100,INFO,"This address has expired.");

  u = extdate;
  str_extdate[fmt_ulong(str_extdate,u)] = 0;
  cookie(hash,key.s,key.len,str_extdate);

  if (*x == '.') ++x;
  if ((str_len(x) != COOKIE) || !byte_equal(hash,COOKIE,x))
    strerr_die2x(100,INFO,"This address is invalid.");
 
  strerr_die2x(0,INFO,"Address is valid and hasn't expired.");
}
